import click
import importlib


@click.command()
@click.argument("filename")
def cli(filename):
    """
    Runs any python script
    """
    click.echo(f"Executing {filename}")
    sName = filename.replace("/", ".")[:-3]
    out = importlib.import_module(sName)



import click, json
from rich.console import Console
from rich.table import Table
from art_engine.engine_factory import ArtEngineFactory
import art_engine.appconfig as config


@click.group()
def cli():
    """
    Generates and checks planned rarities
    """
    pass


@click.command()
def generate():
    """
    Generate rarity configuration file
    """
    factory = ArtEngineFactory(config)
    engine = factory.getEngine()
    engine.setup_engine()
    engine.generate_rarity_config()


@click.command()
def check():
    """
    Compares and outputs real rarities based on DNA database
    """
    factory = ArtEngineFactory(config)
    engine = factory.getEngine()
    engine.setup_engine()

    dnas = []
    trait_counts = {}
    planned_rarities = []
    project_template = None

    if engine.config.engine_type == "simple":
        project_template = config.simple_project_template
    elif engine.config.engine_type == "complex":
        project_template = config.complex_project_template

    try:
        with open(project_template["cache"] + "/metadata.json", "r") as f:
            dnas = json.load(f)
    except FileNotFoundError:
        print("DNA database not found")
        return
    try:
        with open(config.RARITY_JSON_NAME, "r") as file:
            planned_rarities = json.load(file)
    except FileNotFoundError:
        print("Rarity config file not found")
        return

    number_of_items_in_collection = len(dnas)

    for dna in dnas:
        for trait in dna:
            if trait not in trait_counts:
                trait_counts[trait] = {}
            if dna[trait] not in trait_counts[trait]:
                trait_counts[trait][dna[trait]] = 1
            else:
                trait_counts[trait][dna[trait]] += 1

    table = Table()
    table.add_column(
        f"Trait type / Trait value ([yellow]Σ:{number_of_items_in_collection}[/yellow])",
        style="gold3",
    )
    table.add_column("Item count", style="dodger_blue1")
    table.add_column("Planed Weight", style="dodger_blue1")
    table.add_column("Rarity", style="dodger_blue1")

    for k, v in trait_counts.items():
        for x, y in v.items():
            rarity = "{:.2f}".format(y / number_of_items_in_collection * 100)
            rarity = f"{rarity} %"
            planned_rarity = planned_rarities[k][x]
            table.add_row(f"{k} / {x}", str(y), str(planned_rarity), rarity)
    console = Console()
    console.print(table)


cli.add_command(generate)
cli.add_command(check)



import click
import os
import shutil
from art_engine import appconfig
import pathlib


@click.command()
@click.argument("target_directory", type=str)
@click.option(
    "--project-type",
    type=click.Choice(["simple", "complex"], case_sensitive=False),
    required=True,
)
def cli(target_directory, project_type):
    """
    Initializes a new Art Engine project
    """
    click.echo("Init started ...")
    if project_type == "simple":
        config_file_name = "config_files/simple_instance_config.py"
        project_template = appconfig.simple_project_template
    elif project_type == "complex":
        config_file_name = "config_files/complex_instance_config.py"
        project_template = appconfig.complex_project_template

    parent_dir = pathlib.Path(__file__).parent.resolve().parent.resolve()
    instance_config_full_path = os.path.join(parent_dir, config_file_name)

    for dir in project_template:
        os.makedirs(
            os.path.join(target_directory, project_template[dir]),
            exist_ok=True,
        )
    shutil.copy(
        instance_config_full_path, os.path.join(target_directory, "instance_config.py")
    )
    click.echo("Done ...")



from art_engine.simple_engine import SimpleEngine
from art_engine.complex_engine import ComplexEngine


class ArtEngineFactory:
    def __init__(self, config) -> None:
        self.config = config

    def getEngine(self):
        if self.config.engine_type == "simple":
            return SimpleEngine(self.config)
        elif self.config.engine_type == "complex":
            return ComplexEngine(self.config)
        else:
            raise Exception("Error, unknown engine type :(")



import click
from art_engine.engine_factory import ArtEngineFactory
import art_engine.appconfig as config
from rich.console import Console
from rich.table import Table


@click.command()
def cli():
    """
    Gathers and prints out information about your project
    """
    factory = ArtEngineFactory(config)
    engine = factory.getEngine()

    engine.setup_engine()
    console = Console()
    table = Table(show_header=True, header_style="bold yellow")
    table.add_column("Property")
    table.add_column("Value")

    table.add_row("Max possible combinations", str(engine.max_possible_combinations))
    table.add_row("Number of layers", str(len(engine.config.traits)))

    console.print(table)



import os, math, json, time
from PIL import Image
from art_engine.dna_generator import (
    SequentialGenerator,
    RandomGenerator,
    RarityWeightGenerator,
)
from art_engine.appconfig import simple_project_template


class SimpleEngine:
    def __init__(self, config) -> None:
        self.config = config
        self.dnas = []
        self.sprite_configs = []
        self.trait_options = {}
        self.max_possible_combinations = 0
        self.dna_generator = None

    def setup_engine(self):
        self.check_config_file_values()
        self.collect_trait_options()
        self.calculate_max_possible_combinations()

        if self.config.dna_generation_type == 1:
            self.dna_generator = SequentialGenerator()
        if self.config.dna_generation_type == 2:
            self.dna_generator = RandomGenerator()
        if self.config.dna_generation_type == 3:
            self.dna_generator = RarityWeightGenerator()

    def check_config_file_values(self):
        try:
            assert self.config.engine_type == "simple"
        except:
            raise Exception("engine_type must be simple in your instance_config.py")

        try:
            assert self.config.base_image is not None
            assert os.path.isfile(self.config.base_image) == True
        except:
            raise Exception("Check your base_image in your instance_config.py")

        try:
            assert type(self.config.traits) == list
            assert len(self.config.traits) > 0
        except:
            raise Exception(
                "traits in instance_config.py must exists and can not be empty"
            )

        try:
            assert self.config.dna_generation_type in [1, 2, 3]
        except:
            raise Exception(
                "DNA generation type must be 1, 2 or 3 in instance_config.py"
            )

    def collect_trait_options(self):
        self.trait_options = {
            trait_name: [
                os.path.splitext(x)[0]
                for x in os.listdir(
                    simple_project_template["simple_layers"] + "/" + trait_name
                )
            ]
            for trait_name in self.config.traits
        }

    def calculate_max_possible_combinations(self):
        self.max_possible_combinations = math.prod(
            [len(self.trait_options[t]) for t in self.trait_options]
        )

    def generate_dnas(self, collection_size, retries) -> None:
        self.dnas = self.dna_generator.generate_dnas(
            traits=self.config.traits,
            trait_options=self.trait_options,
            collection_size=collection_size,
            retries=retries,
        )

    def save_dnas(self) -> None:
        if len(self.dnas) == 0:
            return
        with open(
            self.config.simple_project_template["cache"] + "/metadata.json", "w"
        ) as f:
            f.write(json.dumps(self.dnas))

    def load_dnas(self) -> None:
        with open(
            self.config.simple_project_template["cache"] + "/metadata.json", "r"
        ) as f:
            self.dnas = json.load(f)

    def generate_sprite_configs(self) -> None:
        for dna in self.dnas:
            sprite_config = []
            for trait_name, trait_value in dna.items():
                sprite_config.append(
                    f"{self.config.simple_project_template['simple_layers']}/{trait_name}/{trait_value}.png"
                )
            self.sprite_configs.append(sprite_config)

    def generate_rarity_config(self):
        rarity_config = {}
        for trait in self.trait_options:
            rarity_config[trait] = {}
            for gene in self.trait_options[trait]:
                rarity_config[trait][gene] = 1

        with open(self.config.RARITY_JSON_NAME, "w", encoding="utf-8") as f:
            json.dump(rarity_config, f, ensure_ascii=False, indent=4)

    def build_metadata(self, id, dna) -> None:
        atributes = []
        for d in dna:
            if hasattr(self.config, "traits_excluded_from_metadata"):
                if d in self.config.traits_excluded_from_metadata:
                    continue
            atributes.append({"trait_type": d, "value": dna[d]})
        metadata = {
            "name": f"{self.config.item_name} #{id}",
            "description": self.config.description,
            "image": f"{self.config.image_url}/{id}.png",
            "date": f"{int(time.time())}",
            "attributes": atributes,
            "engine": self.config.engine_name,
        }
        with open(
            f"{self.config.simple_project_template['built_metadata']}/{id}.json",
            "w",
            encoding="utf-8",
        ) as f:
            json.dump(metadata, f, ensure_ascii=False, indent=4)
        return metadata

    def build_image(self, id, sprite_config) -> None:
        base_image = Image.open(self.config.base_image).convert("RGBA")
        for i, sprite in enumerate(sprite_config):
            next_image = Image.open(sprite).convert("RGBA")
            base_image.paste(next_image, (0, 0), next_image)
        result = base_image.resize(self.config.image_size)
        result.save(
            f"{self.config.simple_project_template['built_images']}/{id}.png", "PNG"
        )



import click
import pkg_resources  # part of setuptools
from rich.console import Console
from rich.table import Table


@click.command()
def cli():
    """
    AgentFUD Art Engine version and project info
    """
    version = pkg_resources.require("agentfud-art-engine")[0].version
    table = Table(show_header=False)
    table.add_row("artengine version", version)
    table.add_row("Python package", "https://pypi.org/project/agentfud-art-engine/")
    table.add_row("Repository", "https://github.com/AgentFUD/agentfud-art-engine")
    table.add_row("Issues", "https://github.com/AgentFUD/agentfud-art-engine/issues")
    table.add_row("News & Contact", "https://twitter.com/AgentFud")
    console = Console()
    console.print(table)



import click
import os
import json
from rich import print


@click.command()
@click.option("--new-url", "-u", required=True, help="New URL for images")
def cli(new_url):
    """
    Updates image url in generated metadata files
    """
    metadata_files = os.listdir("./build/metadata")
    for fname in metadata_files:
        with open(os.path.join("./build/metadata", fname), "r") as jsonfile:
            json_content = json.load(jsonfile)

        tmp = json_content["image"].rsplit("/", 1)
        new_img_url = "/".join([new_url, tmp[1]])
        json_content["image"] = new_img_url

        with open(os.path.join("./build/metadata", fname), "w") as jsonfile:
            json.dump(json_content, jsonfile, ensure_ascii=False, indent=4)

    print(
        f"[gold3]Processed[/gold3] [blue]{len(metadata_files)}[/blue] [gold3]metadata files[/gold3]"
    )



import sys, os

simple_project_template = {
    "scripts": "scripts",
    "cache": "cache",
    "built_images": "build/images",
    "built_metadata": "build/metadata",
    "simple_layers": "layers/simple",
}

complex_project_template = {
    "scripts": "scripts",
    "cache": "cache",
    "built_images": "build/images",
    "built_metadata": "build/metadata",
    "simple_layer": "layers/simple",
    "complex_layer": "layers/complex",
}

RARITY_JSON_NAME = "rarity-config.json"

image_size = (600, 700)

item_name = "Filemon"

description = "Schrödinger's cat who's name is Filemon"

engine_name = "Agent FUD Art Engine v0.1"

"""
We have 3 DNA generation types:
- 1: totally random DNA generation
- 2: random DNA generation based on rarity configuration
- 3 deterministic, all the possible DNA configurations will be generated
"""
dna_generation_type = 1

"""
When you run it multiple times if clear_project is True
then all the previously generated images and metadata files will be removed.
Otherwise files will be overwritten. 
"""

try:
    sys.path.append(os.getcwd())
    from instance_config import *
except ImportError:
    pass



import click
import shutil
import os
from art_engine.engine_factory import ArtEngineFactory
import art_engine.appconfig as config
from rich.progress import track


@click.command()
@click.argument("type", type=click.Choice(["images", "metadata", "both"]))
@click.option("--clean-up/--no-clean-up", "-cl/-ncl", default=False)
def cli(type, clean_up):
    """
    Bakes images and metadata files
    """
    factory = ArtEngineFactory(config)
    engine = factory.getEngine()
    project_template = None

    if engine.config.engine_type == "simple":
        project_template = config.simple_project_template
    elif engine.config.engine_type == "complex":
        project_template = config.complex_project_template

    if clean_up:
        shutil.rmtree("./build", ignore_errors=True)
        for dir in project_template:
            os.makedirs(
                project_template[dir],
                exist_ok=True,
            )

    engine.setup_engine()
    try:
        engine.load_dnas()
    except FileNotFoundError:
        print("DNA database has not been found, please run artengine dna command first")
        return

    engine.generate_sprite_configs()

    if type in ["metadata", "both"]:
        for index, dna in track(
            enumerate(engine.dnas),
            total=len(engine.dnas),
            description="Baking metadata:",
        ):
            engine.build_metadata(index, dna)
    if type in ["images", "both"]:
        for index, sprite_config in track(
            enumerate(engine.sprite_configs),
            total=len(engine.sprite_configs),
            description="Baking images:",
        ):
            engine.build_image(index, sprite_config)



base_image = "./layers/Base.png"

traits = ["Background Color", "Color", "Body", "Ear", "Tail", "Eye", "Mouth", "Foot"]

complex_traits = ["Color"]

colors = {"Greenish": ["Body", "Ear", "Tail"], "Yellowish": ["Body", "Ear", "Tail"]}

available_colors = ["Yellowish", "Greenish"]

traits_excluded_from_metadata = ["Body", "Foot"]

image_size = (600, 700)

item_name = "Filemon"

description = "Schrödinger's cat who's name is Filemon"

"""
We have 3 DNA generation types:
- 1: sequential; deterministic, all the possible DNA configurations will be generated
- 2: random; random DNA generation
- 3: random DNA generation based on rarity configuration
"""
dna_generation_type = 2

# The image url in metadata json files
image_url = "https://nft.anothernow.io/filemon-test/images"

##########################################################################
### WARNING!!! DO NOT EDIT BELOW THIS LINE, IT MIGHT BREAK THE ENGINE. ###
##########################################################################
engine_type = "complex"



import click
from art_engine.engine_factory import ArtEngineFactory
import art_engine.appconfig as config
from rich.console import Console
from rich.table import Table


@click.option(
    "--collection-size",
    "-cs",
    required=True,
    type=int,
    help="How many I should try to generate",
)
@click.option(
    "--retries",
    "-r",
    type=int,
    default=0,
    help="How many additional generation steps should I try",
)
@click.command()
def cli(collection_size, retries):
    """
    Generates DNAs for your project in the cache folder
    """
    factory = ArtEngineFactory(config)
    engine = factory.getEngine()
    engine.setup_engine()
    engine.generate_dnas(collection_size, retries)
    engine.save_dnas()

    console = Console()
    table = Table(show_header=True, header_style="bold yellow")
    table.add_column("Info")
    table.add_column("Value")

    table.add_row("Attempted", str(collection_size))
    table.add_row("Generated", str(len(engine.dnas)))

    console.print(table)



from PIL import Image

import math, random, json, time
import os, sys
from art_engine.dna_generator import (
    SequentialGenerator,
    RandomGenerator,
    RarityWeightGenerator,
)


class ComplexEngine:
    def __init__(self, config) -> None:
        self.config = config
        self.dnas = []
        self.sprite_configs = []
        self.trait_options = {}
        self.max_possible_combinations = 0
        self.dna_generator = None
        self.checker()

    def checker(self):
        """
        Checker checks if config is aligned with layers so info and other commands can be called without a crash.
        If there is a problem it should catch the exception and notify user he has to fix configuration and/or layers.
        """
        # Check if config has traits
        if not hasattr(self.config, "traits"):
            print("You have to define traits in your instance_config.py file")
            sys.exit()
        # Check if trait folders exists
        for trait_name in self.config.traits:
            if trait_name == "Color":
                continue
            if not os.path.isdir("./layers/simple/" + trait_name):
                print(
                    f"Trait: {trait_name} is defined in config but not exists as a directory"
                )
                sys.exit()
        # Check if base image exists
        if not os.path.exists(self.config.base_image):
            print("Don't forget to set config.base_image")

    def setup_engine(self):
        if self.config.dna_generation_type == 1:
            self.dna_generator = SequentialGenerator()
        if self.config.dna_generation_type == 2:
            self.dna_generator = RandomGenerator()
        if self.config.dna_generation_type == 3:
            self.dna_generator = RarityWeightGenerator()
        self.prepare_trait_options()
        self.calculate_max_possible_combinations()

    def calculate_max_possible_combinations(self):
        self.max_possible_combinations = math.prod(
            [len(self.trait_options[t]) for t in self.trait_options]
        )

    def prepare_trait_options(self):
        self.trait_options = {
            trait_name: [
                os.path.splitext(x)[0]
                for x in os.listdir("./layers/simple/" + trait_name)
            ]
            for trait_name in self.config.traits
            if trait_name != "Color"
        }
        self.trait_options["Color"] = self.config.available_colors

    def generate_dnas(self, collection_size, retries) -> None:
        self.dnas = self.dna_generator.generate_dnas(
            traits=self.config.traits,
            trait_options=self.trait_options,
            collection_size=collection_size,
            retries=retries,
        )

    def load_dnas(self) -> None:
        with open(
            self.config.complex_project_template["cache"] + "/metadata.json", "r"
        ) as f:
            self.dnas = json.load(f)

    def save_dnas(self) -> None:
        if len(self.dnas) == 0:
            return
        with open(
            self.config.complex_project_template["cache"] + "/metadata.json", "w"
        ) as f:
            f.write(json.dumps(self.dnas))

    def generate_sprite_configs(self) -> None:
        for dna in self.dnas:
            sprite_config = []
            for trait_name, trait_value in dna.items():
                if trait_name == "Color":
                    for trn in self.config.colors[trait_value]:
                        img_name = dna[trn]
                        sprite_config.append(
                            f"./layers/complex/Color/{trait_value}/{trn}/{img_name}.png"
                        )
                else:
                    sprite_config.append(
                        f"./layers/simple/{trait_name}/{trait_value}.png"
                    )
            self.sprite_configs.append(sprite_config)

    def generate_rarity_config(self):
        print("Generating rarity configuration")
        rarity_config = {}
        for trait in self.trait_options:
            rarity_config[trait] = {}
            for gene in self.trait_options[trait]:
                rarity_config[trait][gene] = 1

        with open(self.config.RARITY_JSON_NAME, "w", encoding="utf-8") as f:
            json.dump(rarity_config, f, ensure_ascii=False, indent=4)
        print("Done...")

    def build_image(self, id, sprite_config) -> None:
        base_image = Image.open(self.config.base_image)
        for i, sprite in enumerate(sprite_config):
            next_image = Image.open(sprite)
            base_image.paste(next_image, (0, 0), next_image)
        result = base_image.resize(self.config.image_size)
        result.save(f"./build/images/{id}.png", "PNG")

    def build_metadata(self, id, dna) -> None:
        atributes = []
        for d in dna:
            if hasattr(self.config, "traits_excluded_from_metadata"):
                if d in self.config.traits_excluded_from_metadata:
                    continue
            atributes.append({"trait_type": d, "value": dna[d]})
        metadata = {
            "name": f"{self.config.item_name} #{id}",
            "description": self.config.description,
            "image": f"{self.config.image_url}/{id}.png",
            "date": f"{int(time.time())}",
            "attributes": atributes,
            "engine": self.config.engine_name,
        }
        with open(f"./build/metadata/{id}.json", "w", encoding="utf-8") as f:
            json.dump(metadata, f, ensure_ascii=False, indent=4)
        return metadata



from setuptools import setup, find_packages
import os, pathlib

DESCRIPTION = 'AgentFUD Art Engine'

this_directory = pathlib.Path(__file__).parent.resolve()
with open(os.path.join(this_directory, 'README.md')) as readme:
    LONG_DESCRIPTION = readme.read()

def read_requirements():
    with open("requirements.txt") as req:
        content = req.read()
        requirements = content.split("\n")
    return requirements


setup(
    name="agentfud-art-engine",
    version="0.3.4",
    author='AgentFUD',
    author_email='agentfud@gmail.com',
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    install_requires=read_requirements(),
    url='https://github.com/AgentFUD/agentfud-art-engine',
    keywords=['python', 'python3', 'Art Engine', 'NFT Generator', 'NFT', 'Image generator'],
    entry_points="""
        [console_scripts]
        artengine=art_engine.cli:cli
    """,
    classifiers=[
       "Programming Language :: Python :: 3",
       "License :: OSI Approved :: MIT License",
       "Operating System :: OS Independent",
   ],
   python_requires='>=3.6'
)



from abc import ABC, abstractmethod
from typing import List, Optional
from itertools import product
import random
import json
import sys
import hashlib
from rich import print
from art_engine.appconfig import RARITY_JSON_NAME


class SimpleDNAGeneratorStrategy(ABC):
    @abstractmethod
    def generate_dnas(
        self,
        traits: List[str],
        trait_options: List[dict],
        collection_size: int,
        retries: Optional[int],
    ) -> List[dict]:
        pass


class RetryDNAGeneratorStrategy(ABC):
    @abstractmethod
    def generate_dnas(
        self,
        traits: List[str],
        trait_options: List[dict],
        collection_size: int,
        retries: int,
    ) -> List[dict]:
        pass


class SequentialGenerator(SimpleDNAGeneratorStrategy):
    def generate_dnas(
        self,
        traits: List[str],
        trait_options: List[dict],
        collection_size: int,
        retries: Optional[int],
    ) -> List[dict]:
        ordered_trait_options = [trait_options[val] for val in traits]
        dnas = []
        for x in product(*ordered_trait_options):
            y = zip(traits, list(x))
            dnas.append(dict(y))
        return dnas[:collection_size]


class RandomGenerator(SimpleDNAGeneratorStrategy):
    def generate_dnas(
        self,
        traits: List[str],
        trait_options: List[str],
        collection_size: int,
        retries: Optional[int],
    ) -> List[dict]:
        ordered_trait_options = [trait_options[val] for val in traits]
        dnas = []
        for x in product(*ordered_trait_options):
            y = zip(traits, list(x))
            dnas.append(dict(y))
        random.shuffle(dnas)
        return dnas[:collection_size]


class RarityWeightGenerator(RetryDNAGeneratorStrategy):
    def hash_dna(self, dna):
        return hashlib.sha1(json.dumps(dna, sort_keys=True).encode("UTF-8")).hexdigest()

    def generate_dnas(
        self,
        traits: List[str],
        trait_options: List[str],
        collection_size: int,
        retries: int,
    ) -> List[dict]:
        try:
            with open(RARITY_JSON_NAME, "r") as file:
                rarities = json.load(file)
        except FileNotFoundError:
            print("-" * 75)
            print(
                f"[gold3]Rarity configuration file[/gold3] [blue]{RARITY_JSON_NAME}[/blue] [gold3]can not be found. Terminating.[/gold3]"
            )
            print("-" * 75)
            sys.exit()

        existing_dna_hashes = []
        dnas = []
        for d in range(collection_size + retries):
            dnax = {}
            for n in traits:
                dnax[n] = random.choices(
                    trait_options[n], weights=rarities[n].values(), k=1
                )[0]
            h = self.hash_dna(dnax)
            if h not in existing_dna_hashes:
                existing_dna_hashes.append(h)
                dnas.append(dnax)
        return dnas[:collection_size]
